@@ -4,7 +4,7 @@ use v5.10;
 use strict;
 use warnings;
 
-our $VERSION = "2.02";
+our $VERSION = "2.03";
 
 use strict;
 use warnings;
@@ -18,17 +18,19 @@ sub vsprintf { &sprintf(@_) }
 
 sub sprintf {
     my($format, @args) = @_;
-    my @list;
 
-    my $uniqstr = _sub_uniqstr();
-    for my $arg (@args) {
-	next if not defined $arg;
-	next if $arg !~ /\P{ASCII}/;
-	push @list, $arg;
-	push @list, $arg = $uniqstr->($arg);
+    my $uniqstr = _sub_uniqstr($format, @args)
+	or return CORE::sprintf($format, @args);
+
+    my @list;
+    for (@args) {
+	defined and /\P{ASCII}/ or next;
+	my $replacement = $uniqstr->($_) // next;
+	push @list, $replacement => $_;
+	$_ = $replacement;
     }
     my $result = CORE::sprintf($format, @args);
-    while (my($orig, $tmp) = splice(@list, 0, 2)) {
+    while (my($tmp, $orig) = splice(@list, 0, 2)) {
 	$result =~ s/$tmp/$orig/;
     }
     $result;
@@ -42,15 +44,24 @@ sub printf {
 use Text::VisualWidth::PP;
 
 sub _sub_uniqstr {
+    my $format = shift;
+    my @seq;
+
+  LOOP:
+    for my $i (1 .. 5) {
+	for my $j (1 .. 5) {
+	    my $seq = pack "CC", $i, $j;
+	    push(@seq, $seq) if index($format, $seq) < 0;
+	    last LOOP if @seq >= @_;
+	}
+    }
+    return undef if @seq == 0;
+
     my $n = 0;
     sub {
-	my $len = Text::VisualWidth::PP::width(shift)
-	    or croak "Unexpected input.";
-	$len == 1 and return "\006";
-	$n > 24 and $n = 24;
-	my $s = pack("CC", $n / 5 + 1, $n % 5 + 1) . ("_" x ($len - 2));
-	$n++;
-	$s;
+	my $len = Text::VisualWidth::PP::width(shift);
+	return undef if $len < 2;
+	CORE::sprintf("%s%s", $seq[$n++ % @seq], "_" x ($len - 2));
     }
 }
 
@@ -103,14 +114,8 @@ Strings in the LIST which contains wide-width character are replaced
 before formatting, and recovered after the process.
 
 Unique replacement string contains a combination of control characters
-(Control-A to Control-E).  So, if the FORMAT contains a string in this
-range, it has a chance to be a subject of replacement.
-
-Single half-width multi-byte character is exception, and all
-represented by single octal 006 (Control-F) character.  It may sounds
-odd, but they are converted to proper string because the order is
-preserved.  Same thing can be done for longer arguments, and when the
-number or arguments exceeds 25, they are encoded by same code.
+(Control-A to Control-E).  If the FORMAT contains all of these two
+bytes combinations, the function behaves just like a standard one.
 
 =head1 SEE ALSO
 
